import React, { useEffect, useState } from "react";
import { uniqueId } from "lodash-es";
import { classNames } from "@microsoft/fast-web-utilities";
/**
 * Custom form control definition
 */
function FileUploadControl(props) {
    /**
     * The id of the file input
     */
    const fileId = uniqueId();
    /**
     * File reader to handle reading / parsing of file data
     */
    const reader = new FileReader();
    const [dragging, setDragging] = useState(false);
    const [processing, setProcessing] = useState(false);
    useEffect(() => {
        reader.addEventListener("load", handleReaderLoad);
        return function cleanup() {
            reader.removeEventListener("load", handleReaderLoad);
        };
    });
    /**
     * Callback to call when the reader loads file data
     */
    function handleReaderLoad() {
        setProcessing(false);
        props.onChange({ value: reader.result });
    }
    /**
     * Event handler that effectively cancels the event
     */
    function cancelEvent(e) {
        e.preventDefault();
    }
    /**
     * Updates the component with new file data
     */
    function updateWithFile(file) {
        // We should exit if file is not actually a File or the type is not an image
        if (!(file instanceof File) || !file.type.includes("image")) {
            return;
        }
        setProcessing(true);
        reader.readAsDataURL(file);
    }
    /**
     * Event handler for drag-area dragover event
     */
    function dragOver(e) {
        cancelEvent(e);
        if (!dragging) {
            setDragging(true);
        }
    }
    /**
     * Event handler for drag-area drag-leave event
     */
    function dragLeave(e) {
        cancelEvent(e);
        setDragging(false);
    }
    /**
     * Callback for drop event
     */
    function onDrop(e) {
        cancelEvent(e);
        setDragging(false);
        updateWithFile(e.dataTransfer.files[0]);
    }
    /**
     * Callback for input change
     */
    function handleInputOnChange(e) {
        updateWithFile(e.target.files[0]);
    }
    function generateStaticUI() {
        return [
            typeof props.value === "string" ? (React.createElement("img", { key: "thumbnail", src: props.value, alt: props.strings.fileUploadPreviewAlt })) : null,
            React.createElement("input", { key: "input", className: "dtc-file-upload-control_input", type: "file", id: fileId, onChange: handleInputOnChange }),
            React.createElement("p", { key: "info" },
                props.strings.fileUploadDragInstr,
                " ",
                React.createElement("label", { htmlFor: fileId }, props.strings.fileUploadBrowseFiles)),
        ];
    }
    function generateProcessingUI() {
        return React.createElement("p", null, props.strings.fileUploadUploading);
    }
    return (React.createElement("div", { className: classNames("dtc-file-upload-control", [
            "dtc-file-upload-control__disabled",
            props.disabled,
        ]), onDragEnter: cancelEvent, onDragOver: dragOver, onDragLeave: dragLeave, onDrop: onDrop }, processing ? generateProcessingUI() : generateStaticUI()));
}
export { FileUploadControl };
export default FileUploadControl;
