import { ArrayControl, ButtonControl, CheckboxControl, DisplayControl, LinkedDataControl, NumberFieldControl, SectionControl, SectionLinkControl, SelectControl, TextareaControl, } from "./controls";
import { BareControlPlugin, ControlType, SingleLineControlPlugin, StandardControlPlugin, } from "./templates";
import { ControlContext, LinkedDataActionType } from "./templates/types";
import { getDictionaryBreadcrumbs } from "./utilities";
import { cloneDeep, get } from "lodash-es";
import React, { useEffect, useState } from "react";
import defaultStrings from "./form.strings";
import { classNames } from "@microsoft/fast-web-utilities";
import { dataSetName, MessageSystemDataTypeAction, MessageSystemNavigationTypeAction, MessageSystemType, } from "design-to-code";
import cleanListStyle from "design-to-code/dist/stylesheets/web-components/style/common.clean-list.css";
import style from "./form.style.css";
import dtcClassName from "design-to-code/dist/esm/web-components/style/class-names";
// tree-shaking
cleanListStyle;
style;
export const formId = "dtc-react::form";
/**
 * Schema form component definition
 */
const Form = (props = {
    controls: [],
    displayValidationBrowserDefault: true,
}) => {
    /**
     * The default untitled string
     */
    const untitled = "Untitled";
    /**
     * The default form controls
     */
    let selectControl;
    let displayControl;
    let linkedDataControl;
    let sectionLinkControl;
    let sectionControl;
    let checkboxControl;
    let numberFieldControl;
    let textareaControl;
    let arrayControl;
    let buttonControl;
    /**
     * The default form components as a dictionary
     * by type
     */
    const controlComponents = {};
    let strings = !!props.strings ? props.strings : defaultStrings;
    updateControls();
    const [activeDictionaryId, setActiveDictionaryId] = useState("");
    const [activeNavigationConfigId, setActiveNavigationConfigId] = useState("");
    const [data, setData] = useState(void 0);
    const [dataDictionary, setDataDictionary] = useState(void 0);
    const [schemaDictionary, setSchemaDictionary] = useState({});
    const [navigationDictionary, setNavigationDictionary] = useState(void 0);
    const [validationErrors, setValidationErrors] = useState({});
    const [options, setOptions] = useState(null);
    useEffect(() => {
        const messageSystemConfig = {
            onMessage: handleMessageSystem,
            config: {
                displayTextDataLocation: dataSetName,
            },
        };
        if (props.messageSystem !== undefined) {
            props.messageSystem.add(messageSystemConfig);
        }
        return function cancel() {
            if (props.messageSystem !== undefined) {
                props.messageSystem.remove(messageSystemConfig);
            }
        };
    });
    function renderForm() {
        return navigationDictionary ? (React.createElement("form", { onSubmit: handleSubmit },
            renderBreadcrumbs(),
            renderSection())) : null;
    }
    /**
     * Handle messages from the message system
     */
    function handleMessageSystem(e) {
        var _a;
        let setState = false;
        let updatedActiveDictionaryId = e.data.activeDictionaryId;
        switch (e.data.type) {
            case MessageSystemType.initialize:
            case MessageSystemType.data:
                updatedActiveDictionaryId = ((_a = e.data) === null || _a === void 0 ? void 0 : _a.activeDictionaryId)
                    ? e.data.activeDictionaryId
                    : activeDictionaryId;
            case MessageSystemType.navigation:
            case MessageSystemType.validation:
            case MessageSystemType.schemaDictionary:
                setState = true;
                break;
        }
        if (setState) {
            const updatedSchemaDictionary = e.data.schemaDictionary;
            let updatedDataDictionary = e.data.dataDictionary;
            let updatedNavigationDictionary = e.data.navigationDictionary;
            let updatedActiveDictionaryId = e.data.activeDictionaryId;
            let updatedActiveNavigationConfigId = e.data.activeNavigationConfigId;
            let updatedValidationErrors = e.data.validation;
            let updatedData = updatedDataDictionary[0][updatedActiveDictionaryId].data;
            let updatedOptions = e.data.options;
            setSchemaDictionary(updatedSchemaDictionary);
            setDataDictionary(updatedDataDictionary);
            setActiveDictionaryId(updatedActiveDictionaryId);
            setActiveNavigationConfigId(updatedActiveNavigationConfigId);
            setNavigationDictionary(updatedNavigationDictionary);
            setValidationErrors(updatedValidationErrors);
            setOptions(updatedOptions);
            setData(updatedData);
        }
    }
    /**
     * Find all type controls passed and use them in
     * place of the default controls
     */
    function findControlPlugin(hasCustomControlPlugins, type) {
        const controlPluginConfig = getComponentByType(type);
        if (hasCustomControlPlugins) {
            const controlPlugin = props.controls.find((control) => {
                return control.matchesType(type);
            });
            if (controlPlugin !== undefined) {
                controlComponents[type] =
                    controlPlugin.config.component !== undefined
                        ? controlPlugin.config.component
                        : controlPluginConfig.component;
                return controlPlugin;
            }
            const allControlsPlugin = props.controls.find((control) => {
                return control.matchesAllTypes();
            });
            if (allControlsPlugin !== undefined) {
                controlComponents[type] =
                    allControlsPlugin.config.component !== undefined
                        ? allControlsPlugin.config.component
                        : controlPluginConfig.component;
                return allControlsPlugin;
            }
        }
        const ControlComponent = controlPluginConfig.component;
        controlComponents[type] = ControlComponent;
        return new controlPluginConfig.plugin(Object.assign(Object.assign({}, controlPluginConfig), { type, control: (config) => {
                return React.createElement(ControlComponent, Object.assign({}, config));
            } }));
    }
    function getComponentByType(type) {
        switch (type) {
            case ControlType.select:
                return {
                    plugin: StandardControlPlugin,
                    component: SelectControl,
                    context: ControlContext.fill,
                };
            case ControlType.array:
                return {
                    plugin: StandardControlPlugin,
                    component: ArrayControl,
                    context: ControlContext.fill,
                };
            case ControlType.linkedData:
                return {
                    plugin: StandardControlPlugin,
                    component: LinkedDataControl,
                    context: ControlContext.fill,
                };
            case ControlType.numberField:
                return {
                    plugin: StandardControlPlugin,
                    component: NumberFieldControl,
                    context: ControlContext.fill,
                };
            case ControlType.checkbox:
                return {
                    plugin: SingleLineControlPlugin,
                    component: CheckboxControl,
                    context: ControlContext.default,
                };
            case ControlType.sectionLink:
                return {
                    plugin: StandardControlPlugin,
                    component: SectionLinkControl,
                    context: ControlContext.fill,
                };
            case ControlType.textarea:
                return {
                    plugin: StandardControlPlugin,
                    component: TextareaControl,
                    context: ControlContext.fill,
                };
            case ControlType.display:
                return {
                    plugin: StandardControlPlugin,
                    component: DisplayControl,
                    context: ControlContext.fill,
                };
            case ControlType.button:
                return {
                    plugin: StandardControlPlugin,
                    component: ButtonControl,
                    context: ControlContext.fill,
                };
            default:
                return {
                    plugin: BareControlPlugin,
                    component: SectionControl,
                    context: ControlContext.default,
                };
        }
    }
    function updateControls() {
        const hasCustomControlPlugins = Array.isArray(props.controls);
        selectControl = findControlPlugin(hasCustomControlPlugins, ControlType.select);
        arrayControl = findControlPlugin(hasCustomControlPlugins, ControlType.array);
        linkedDataControl = findControlPlugin(hasCustomControlPlugins, ControlType.linkedData);
        numberFieldControl = findControlPlugin(hasCustomControlPlugins, ControlType.numberField);
        checkboxControl = findControlPlugin(hasCustomControlPlugins, ControlType.checkbox);
        sectionLinkControl = findControlPlugin(hasCustomControlPlugins, ControlType.sectionLink);
        textareaControl = findControlPlugin(hasCustomControlPlugins, ControlType.textarea);
        displayControl = findControlPlugin(hasCustomControlPlugins, ControlType.display);
        buttonControl = findControlPlugin(hasCustomControlPlugins, ControlType.button);
        sectionControl = findControlPlugin(hasCustomControlPlugins, ControlType.section);
    }
    /**
     * Generates the breadcrumb navigation
     */
    function renderBreadcrumbs() {
        const breadcrumbs = getDictionaryBreadcrumbs(navigationDictionary, activeDictionaryId, activeNavigationConfigId, handleBreadcrumbClick, untitled);
        if (breadcrumbs.length > 1) {
            return (React.createElement("ul", { className: `dtc-form_breadcrumbs ${dtcClassName.commonCleanList}` }, renderBreadcrumbItems(breadcrumbs)));
        }
    }
    function renderBreadcrumbItems(items) {
        return items.map((item, index) => {
            if (index === items.length - 1) {
                return (React.createElement("li", { key: index },
                    React.createElement("span", null, item.text || untitled)));
            }
            return (React.createElement("li", { key: index },
                React.createElement("a", { href: item.href, onClick: item.onClick }, item.text || untitled)));
        });
    }
    /**
     * Render the section to be shown
     */
    function renderSection() {
        let control = sectionControl;
        const navigationItem = navigationDictionary[0][activeDictionaryId][0][activeNavigationConfigId];
        // Check to see if there is any associated `formControlId`
        // then check for the id within the passed controlPlugins
        if (typeof get(navigationItem, `schema.formControlId`) === "string") {
            control = props.controls.find((controlPlugin) => {
                return controlPlugin.matchesId(navigationItem.schema.formControlId);
            });
            if (control === undefined) {
                control = sectionControl;
            }
        }
        control.updateProps({
            index: 0,
            type: ControlType.section,
            required: false,
            label: navigationItem.text || untitled,
            invalidMessage: "",
            component: controlComponents[ControlType.section],
            schema: navigationItem.schema,
            schemaDictionary,
            controls: {
                button: buttonControl,
                array: arrayControl,
                linkedData: linkedDataControl,
                checkbox: checkboxControl,
                display: displayControl,
                textarea: textareaControl,
                select: selectControl,
                section: sectionControl,
                sectionLink: sectionLinkControl,
                numberField: numberFieldControl,
            },
            controlPlugins: props.controls,
            controlComponents: controlComponents,
            onChange: handleOnChange,
            onUpdateSection: handleUpdateActiveSection,
            data: navigationItem.data,
            schemaLocation: navigationItem.schemaLocation,
            default: get(navigationItem, "schema.default"),
            disabled: navigationItem.disabled,
            dataLocation: navigationDictionary[0][activeDictionaryId][0][navigationItem.self]
                .relativeDataLocation,
            navigationConfigId: navigationItem.self,
            dictionaryId: activeDictionaryId,
            dataDictionary: dataDictionary,
            navigation: navigationDictionary[0][activeDictionaryId][0],
            untitled: untitled,
            validationErrors: validationErrors[activeDictionaryId],
            displayValidationBrowserDefault: props.displayValidationBrowserDefault,
            displayValidationInline: props.displayValidationInline,
            displayValidationErrorList: props.displayValidationErrorList,
            messageSystem: props.messageSystem,
            strings: strings,
            messageSystemOptions: options,
            categories: props.categories || {},
        });
        return control.render();
    }
    function handleBreadcrumbClick(dictionaryId, navigationConfigId) {
        return (e) => {
            e.preventDefault();
            handleUpdateActiveSection(dictionaryId, navigationConfigId);
        };
    }
    function handleOnChange(config) {
        if (props.messageSystem) {
            if (config.isLinkedData) {
                if (config.linkedDataAction === LinkedDataActionType.add) {
                    props.messageSystem.postMessage({
                        type: MessageSystemType.data,
                        action: MessageSystemDataTypeAction.addLinkedData,
                        dataLocation: config.dataLocation,
                        linkedData: config.value,
                        options: {
                            originatorId: formId,
                        },
                    });
                }
                else if (config.linkedDataAction === LinkedDataActionType.reorder) {
                    props.messageSystem.postMessage({
                        type: MessageSystemType.data,
                        action: MessageSystemDataTypeAction.reorderLinkedData,
                        dataLocation: config.dataLocation,
                        linkedData: config.value,
                        options: {
                            originatorId: formId,
                        },
                    });
                }
                else if (config.linkedDataAction === LinkedDataActionType.remove) {
                    props.messageSystem.postMessage({
                        type: MessageSystemType.data,
                        action: MessageSystemDataTypeAction.removeLinkedData,
                        dataLocation: config.dataLocation,
                        linkedData: config.value,
                        options: {
                            originatorId: formId,
                        },
                    });
                }
            }
            else if (config.isArray) {
                const updatedData = config.dataLocation === ""
                    ? cloneDeep(data)
                    : cloneDeep(get(data, config.dataLocation)) || [];
                let newArray;
                if (typeof config.index !== "undefined") {
                    newArray = updatedData.filter((item, itemIndex) => {
                        return itemIndex !== config.index;
                    });
                }
                else {
                    newArray = updatedData;
                    newArray.push(config.value);
                }
                props.messageSystem.postMessage({
                    type: MessageSystemType.data,
                    action: MessageSystemDataTypeAction.update,
                    dataLocation: config.dataLocation,
                    data: newArray,
                    options: {
                        originatorId: formId,
                    },
                });
            }
            else {
                if (config.value === undefined) {
                    props.messageSystem.postMessage({
                        type: MessageSystemType.data,
                        action: MessageSystemDataTypeAction.remove,
                        dataLocation: config.dataLocation,
                        options: {
                            originatorId: formId,
                        },
                    });
                }
                else {
                    props.messageSystem.postMessage({
                        type: MessageSystemType.data,
                        action: MessageSystemDataTypeAction.update,
                        dataLocation: config.dataLocation,
                        data: config.value,
                        options: {
                            originatorId: formId,
                        },
                    });
                }
            }
        }
    }
    /**
     * Handles the form submit
     */
    function handleSubmit(e) {
        e.preventDefault();
    }
    /**
     * Handles an update to the active section and component
     */
    function handleUpdateActiveSection(dictionaryId, navigationConfigId) {
        if (props.messageSystem) {
            props.messageSystem.postMessage({
                type: MessageSystemType.navigation,
                action: MessageSystemNavigationTypeAction.update,
                activeNavigationConfigId: navigationConfigId !== undefined
                    ? navigationConfigId
                    : navigationDictionary[0][dictionaryId][1],
                activeDictionaryId: dictionaryId,
                options: {
                    originatorId: formId,
                },
            });
        }
    }
    return React.createElement("div", { className: classNames("dtc-form") }, renderForm());
};
export default Form;
